// LineClient.java

import java.io.*;
import java.net.*;

public class LineClient {

    public static void main(String[] args) {

        InetAddress addr = null;
        int port = -1;

        try {
            if (args.length == 2) {
                addr = InetAddress.getByName(args[0]);
                port = Integer.parseInt(args[1]);
            } else {
                System.out.println("Usage: java LineClient serverIP serverPort");
                System.exit(1);
            }
        } catch (UnknownHostException e) {
            System.out.println("Error while resolving server: ");
            e.printStackTrace();
            System.out.println("LineClient: terminating.");
            System.exit(2);
        }

        DatagramSocket socket = null;
        DatagramPacket packet = null;
        byte[] buf = new byte[512];

        // create datagram socket, set 30s timeout
        // and create datagram packet
        try {
            socket = new DatagramSocket();
            socket.setSoTimeout(30000);
            packet = new DatagramPacket(buf, buf.length, addr, port);
            System.out.println("\nLineClient: started");
            System.out.println("Socket created: " + socket);

        } catch (SocketException e) {
            System.out.println("Error while creating socket: ");
            e.printStackTrace();
            System.out.println("LineClient: terminating.");
            System.exit(1);
        }

        BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));
        System.out.print("\n(^D(Unix)/^Z(Win)+enter to quit) Please input a complete (with extension) filename: ");

        try {
            ByteArrayOutputStream boStream = null;
            DataOutputStream doStream = null;
            byte[] data = null;
            String filename = null;
            int lineNumber = -1;
            String response = null;
            ByteArrayInputStream biStream = null;
            DataInputStream diStream = null;

            while ((filename = stdIn.readLine()) != null) {
                // User input
                try {
                    System.out.print("Line number? ");
                    lineNumber = Integer.parseInt(stdIn.readLine());
                } catch (Exception e) {
                    System.out.println("Error while reading from console: ");
                    e.printStackTrace();
                    System.out.print("\n(^D(Unix)/^Z(Win)+enter to quit) Please input a complete (with extension) filename: ");
                    continue;
                    // client repeats loop
                }

                // build packet, fill it and send it
                try {
                    boStream = new ByteArrayOutputStream();
                    doStream = new DataOutputStream(boStream);
                    doStream.writeInt(lineNumber);
                    doStream.writeUTF(filename);
                    doStream.flush();
                    data = boStream.toByteArray();
                    packet.setData(data);
                    socket.send(packet);
                    System.out.println("Request sent to " + addr + ", port " + port);
                } catch (IOException e) {
                    System.out.println("Error while sending request: ");
                    e.printStackTrace();
                    System.out.print("\n(^D(Unix)/^Z(Win)+enter to quit) Please input a complete (with extension) filename: ");
                    continue;
                }

                try {
                    // set input buffer
                    packet.setData(buf);
                    socket.receive(packet);
                    // blocking until the timeout runs out, after it throws a SocketException
                } catch (IOException e) {
                    System.out.println("Problemi nella ricezione del datagramma: ");
                    e.printStackTrace();
                    System.out.print("\n(^D(Unix)/^Z(Win)+enter to quit) Please input a complete (with extension) filename: ");
                    continue;
                    // client repeats loop
                }
                try {
                    biStream = new ByteArrayInputStream(packet.getData(), 0, packet.getLength());
                    diStream = new DataInputStream(biStream);
                    response = diStream.readUTF();
                    System.out.println("Response: " + response);
                } catch (IOException e) {
                    System.out.println("Error while reading response: ");
                    e.printStackTrace();
                    System.out.print("\n(^D(Unix)/^Z(Win)+enter to quit) Please input a complete (with extension) filename: ");
                    continue;
                    // client repeats loop
                }

                // ask for another file
                System.out.print("\n(^D(Unix)/^Z(Win)+enter to quit) Please input a complete (with extension) filename: ");
            } // while
        } // catch all exceptions not managed in the main loop and terminate
        catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println("LineClient: terminating.");
        socket.close();
    }
}
