// LineServer.java

import java.io.*;
import java.net.*;

public class LineServer {

    // Listening port must be in the unprivileged range [1024-65535!
    // declared as stetic because
    private static final int PORT = 4242;

    public static void main(String[] args) {

        System.out.println("LineServer: started");

        DatagramSocket socket = null;
        DatagramPacket packet = null;
        byte[] buf = new byte[256];
        int port = -1;

        // check input arguments: either 0 or 1 argoment (port number)
        if ((args.length == 0)) {
            port = PORT;
        } else if (args.length == 1) {
            try {
                port = Integer.parseInt(args[0]);
                // check that the port is in the unprivileged range
                if (port < 1024 || port > 65535) {
                    System.out.println("Usage: java LineServer [serverPort>=1024]");
                    System.exit(1);
                }
            } catch (NumberFormatException e) {
                System.out.println("Usage: java LineServer [serverPort>=1024]");
                System.exit(1);
            }
        } else {
            System.out.println("Usage: java LineServer [serverPort>=1024]");
            System.exit(1);
        }

        try {
            socket = new DatagramSocket(port);
            packet = new DatagramPacket(buf, buf.length);
            System.out.println("Built socket: " + socket);
        } catch (SocketException e) {
            System.out.println("Error while creating socket: ");
            e.printStackTrace();
            System.exit(1);
        }

        try {

            String filename = null;
            int linenumber = -1;
            ByteArrayInputStream biStream = null;
            DataInputStream diStream = null;
            ByteArrayOutputStream boStream = null;
            DataOutputStream doStream = null;
            String line = null;
            byte[] data = null;

            while (true) {
                System.out.println("\nWaiting for requests");

                // ricezione del datagramma
                try {
                    packet.setData(buf);
                    socket.receive(packet);
                } catch (IOException e) {
                    System.err.println("Error while receiving packet: " + e.getMessage());
                    e.printStackTrace();
                    continue;
                    // server repeats endless loop
                }

                try {
                    biStream = new ByteArrayInputStream(packet.getData(), 0, packet.getLength());
                    diStream = new DataInputStream(biStream);
                    linenumber = diStream.readInt();
                    filename = diStream.readUTF();
                    System.out.println("Requested line " + linenumber + " from file " + filename);
                } catch (Exception e) {
                    System.err.println("Error while reading request: " + filename + " " + linenumber);
                    e.printStackTrace();
                    continue;
                    // server repeats endless loop
                }

                // read line, build response and send it
                try {
                    line = LineUtility.getLine(filename, linenumber);
                    boStream = new ByteArrayOutputStream();
                    doStream = new DataOutputStream(boStream);
                    doStream.writeUTF(line);
                    data = boStream.toByteArray();
                    packet.setData(data, 0, data.length);
                    socket.send(packet);
                } catch (IOException e) {
                    System.err.println("Error while sending response: " + e.getMessage());
                    e.printStackTrace();
                    continue;
                    // server repeats endless loop
                }

            } // while

        } // catch all exceptions not managed in the main loop and terminate
        catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("LineServer: termino...");
        socket.close();
    }
}
